 /*
 *  file      : poche_txt.c
 *  project   : zoomc
 *  with      : Gtk-2
 *
 *  copyright : (C) 2010 - 2011 by Claude Bulin
 *
 *  ZOOMC - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of ZOOMC.
 * 
 *  ZOOMC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  ZOOMC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with ZOOMC.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <glib/gprintf.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>	// M_PI
#include <cairo.h>
#include <cairo-pdf.h>
#include <cairo-ps.h>
#include <cairo-xlib.h>
#include <X11/Xlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../libzoomc/lib.h"
#endif

#include "global.h"
#include "poche.h"



typedef struct {
	gboolean	BoolAccessChanged;
	GtkWidget	*AdrEntrySearchImgWeb;
	GtkWidget	*AdrEntryNameFileSave;
} VAR_POCHE_TXT;

VAR_POCHE_TXT VarPocheTxt = {
			TRUE,		// BoolAccessChanged
			NULL		// AdrEntrySearchImgWeb
			};


// 
// 
gboolean PocheTxt_get_is_bold( void )
{
	return( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(GLADE_GET_OBJECT("togglebutton_font_bold"))) );
}
void on_togglebutton_font_bold_clicked( GtkButton *button, gpointer user_data )
{
	IMAGE	*Image = poche_get_struct_selected_is_txt();
	
	if( NULL != Image ) {
		Image->BoolFontBold = PocheTxt_get_is_bold();
		gtk_widget_queue_draw( view.AdrDrawingarea );
	}
}
// 
// 
gboolean PocheTxt_get_is_italic( void )
{
	return( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(GLADE_GET_OBJECT("togglebutton_font_italic"))) );
}
void on_togglebutton_font_italic_clicked( GtkButton *button, gpointer user_data )
{
	IMAGE	*Image = poche_get_struct_selected_is_txt();
	
	if( NULL != Image ) {
		Image->BoolFontItalic = PocheTxt_get_is_italic();
		gtk_widget_queue_draw( view.AdrDrawingarea );
	}
}
	
	
// 
// 
gchar *PocheTxt_get_ptr_entry_name_file_to_save( void )
{
	if( NULL != VarPocheTxt.AdrEntryNameFileSave ) {
		return( (gchar *)gtk_entry_get_text (GTK_ENTRY (VarPocheTxt.AdrEntryNameFileSave)));
	}
	return( NULL );
}
void on_entry_name_save_file_changed( GtkEditable *editable, gpointer user_data )
{
	if( NULL != VarPocheTxt.AdrEntryNameFileSave ) {
		poche_set_flag_buttons();
	}
}
void on_entry_name_save_file_realize( GtkWidget *widget, gpointer user_data )
{
	VarPocheTxt.AdrEntryNameFileSave = widget;
}



// 
// 
gchar *PocheTxt_get_ptr_entry_img_web( void )
{
	if( NULL != VarPocheTxt.AdrEntrySearchImgWeb ) {
		return( (gchar *)gtk_entry_get_text (GTK_ENTRY (VarPocheTxt.AdrEntrySearchImgWeb)));
	}
	return( NULL );
}
// 
// 
void on_entry_img_web_changed( GtkEditable *editable, gpointer user_data )
{
	if( NULL != VarPocheTxt.AdrEntrySearchImgWeb ) {
		poche_set_flag_buttons ();
	}
}
void on_entry_img_web_realize( GtkWidget *widget, gpointer user_data )
{
	VarPocheTxt.AdrEntrySearchImgWeb = widget;
}

// LE TITRE S'AFFICHE DE FACON PERPENDICULARE SUR LES COTES DE LA POCHETTE
// 
void on_entry_titre_changed( GtkEditable *editable, gpointer user_data )
{
	if( NULL != (gchar *)gtk_entry_get_text (GTK_ENTRY (editable))) {
		poche_set_texte_title( (gchar *)gtk_entry_get_text (GTK_ENTRY (editable)) );
	}
}
void on_entry_titre_realize( GtkWidget *widget, gpointer user_data )
{
}


// INSERTION D UNE NOUVELLE BOITE DE TEXTE
// 
void on_button_text_add_clicked( GtkButton *button, gpointer user_data )
{
	IMAGE		*Image = NULL;
	
	Image = poche_add_to_glist( NULL, 100, 100, TRUE, _TYPE_TEXT_ );
	poche_set_selected_flag_image( Image );
	gtk_widget_queue_draw( view.AdrDrawingarea );
	PocheTxt_set_text_to_textview( Image );
}
// 
// 
void on_button_import_title_cd_clicked( GtkButton *button, gpointer user_data )
{
	PRINT("");
	// 
}
// 
// 
void on_button_import_img_web_clicked( GtkButton *button, gpointer user_data )
{
	PRINT("");
	pocheweb_get();
}
// 
// 
void on_button_import_datas_cd_clicked( GtkButton *button, gpointer user_data  )
{
	IMAGE	*Image = poche_get_struct_selected_is_txt();
	
	if( NULL != Image ) {
		if( NULL != Image->Texte ) {
			g_free( Image->Texte );
			Image->Texte = NULL;
		}
		Image->Texte = g_strdup(
					"01 california ][ 05:01\n"
					"02 XXL ][ 04:28\n"
					"03 l'instant X ][ 04:45\n"
					"04 comme j'ai mal ][ 03:57\n"
					"05 rever ][ 05:25\n"
					"06 l'âme-stram-gram ][ 04:26\n"
					"07 je te rends ton amour ][ 05:13\n"
					"08 souviens toi du jour ][ 05:01\n"
					"09 optimistique moi ][ 04:23\n"
					"10 innamoramento ][ 05:15\n"
					"11 l'histoire dune fee , c'est... ][ 05:02\n"
					);
		PocheTxt_set_text_to_textview( Image );
		gtk_widget_queue_draw( view.AdrDrawingarea );
	}
}
// 
// 
void on_button_font_color_clicked( GtkButton *button, gpointer user_data )
{
	PRINT("");
	// 
}
// 
// 
void on_button_change_font_clicked( GtkButton *button, gpointer user_data )
{
	IMAGE	*Image = NULL;
	
	if( NULL != ( Image = (IMAGE *)poche_get_struct_selected_is_txt())) {
	
		GtkResponseType	result;
		GtkWidget	*dialog = NULL;
		gchar		*Ptr = NULL;
		gchar		*Font = g_strdup_printf( "%s %d", Image->FontName, Image->SizeFontName );
	
		dialog = gtk_font_selection_dialog_new("Select Font");
		gtk_font_selection_dialog_set_font_name( GTK_FONT_SELECTION_DIALOG(dialog), Font );
		g_free( Font );
		Font = NULL;
		result = gtk_dialog_run(GTK_DIALOG(dialog));
		if (result == GTK_RESPONSE_OK || result == GTK_RESPONSE_APPLY) {

			if( NULL != Image->FontName ) {
				g_free( Image->FontName );
				Image->FontName = NULL;
			
			}
			Image->FontName = gtk_font_selection_dialog_get_font_name( GTK_FONT_SELECTION_DIALOG(dialog));
		
			// LE NOM DE LA POLICE: FontName
			Ptr = Image->FontName;
			while( *Ptr ++ );
			Ptr --;
			while( *Ptr != ' ' ) Ptr --;
			Ptr ++;
		
			// LA TAILLE DE LA POLICE: SizeFont
			Image->SizeFontName = atoi( Ptr );
			*Ptr = '\0';
			
			// g_print( "%s %d\n", Image->FontName, Image->SizeFontName );
			
			// FORCER AFFICHAGE
			gtk_widget_queue_draw( view.AdrDrawingarea );
		}
		gtk_widget_destroy(dialog);
	}
}


// COPY BOX TEXT IN TEXTVIEW
// 
void PocheTxt_set_text_to_textview( IMAGE *p_Image )
{
	if( _TYPE_TEXT_ == p_Image->TypeImage ) {
		
		GtkTextBuffer	*Buf = NULL;
		GtkTextIter	Start;
		GtkTextIter	End;
		
		VarPocheTxt.BoolAccessChanged = FALSE;
		
		// ACQUISITION BUFFER TEXTVIEW
		Buf = gtk_text_view_get_buffer( GTK_TEXT_VIEW(view.AdrTextview));
		gtk_text_buffer_get_bounds( Buf, &Start, &End );
		
		// DESTROY BUFFER TEXTE
		gtk_text_buffer_delete( Buf, &Start, &End );
		
		// AFFICHAGE
		if( NULL == p_Image->Texte )
			gtk_text_buffer_set_text( Buf, "", -1 );
		else	gtk_text_buffer_set_text( Buf, p_Image->Texte, -1 );

		VarPocheTxt.BoolAccessChanged = TRUE;
	}
}
// TEXTVIEW -> CHANGED
// 
void PocheTxt_texte_changed_in_textview (GtkTextBuffer *textbuffer, gpointer user_data)
{
	IMAGE		*Image = NULL;
	
	// SI _TYPE_TEXT_ EN SELECTION
	if( TRUE == VarPocheTxt.BoolAccessChanged && NULL != ( Image = poche_get_struct_selected_is_txt())) {
		
		GtkTextBuffer	*Buf = NULL;
		gchar		*PtrTexte = NULL;
		GtkTextIter	Start;
		GtkTextIter	End;
		
		// ACQUISITION BUFFER TEXTVIEW
		Buf = gtk_text_view_get_buffer( GTK_TEXT_VIEW(view.AdrTextview));
		gtk_text_buffer_get_bounds( Buf, &Start, &End );
		
		// ACQUISITION DU TEXTE
		PtrTexte = gtk_text_buffer_get_text (Buf, &Start, &End, FALSE);
		
		// DESTRUCTION BUFFER ACTUEL DE LA BOITE
		if( NULL != Image->Texte ) {
			g_free( Image->Texte );
			Image->Texte = NULL;
		}
		// INSERTION DU NOUVEA TEXTE
		Image->Texte = g_strdup( PtrTexte );
		
		// FORCER AFFICHAGE
		gtk_widget_queue_draw( view.AdrDrawingarea );
	
		g_free( PtrTexte );
		PtrTexte = NULL;
	}
}
// TEXTVIEW -> REALIZE
// 
void on_textview_poche_realize( GtkWidget *widget, gpointer user_data )
{
	view.AdrTextview = widget;
	// g_signal_connect( G_OBJECT(widget),  "changed", G_CALLBACK (PocheTxt_texte_changed_in_textview), NULL);
	g_signal_connect((gpointer) gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget)), "changed", G_CALLBACK (PocheTxt_texte_changed_in_textview), NULL);
}










